using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace BackBufferDemo
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        const string TEXT = "Hello?";
        SpriteFont segoe14;
        Vector2 textPosition;
        Texture2D texture;
        Vector2 texturePosition;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            TargetElapsedTime = TimeSpan.FromTicks(333333);

            // graphics.IsFullScreen = true;

            // OR:

            // graphics.PreferredBackBufferWidth = 728;
            // graphics.PreferredBackBufferHeight = 480;

            // OR:

            // graphics.PreferredBackBufferWidth = 480;
            // graphics.PreferredBackBufferHeight = 768;
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            Viewport viewport = this.GraphicsDevice.Viewport;
            segoe14 = this.Content.Load<SpriteFont>("Segoe14");
            Vector2 textSize = segoe14.MeasureString(TEXT);
            textPosition = 
                new Vector2((int)((viewport.Width - textSize.X) / 2),
                            (int)((viewport.Height - textSize.Y) / 2));

            int radius = (int)(textSize.X / 2 + 1);
            texture = new Texture2D(this.GraphicsDevice, 2 * radius + 1, 
                                                         2 * radius + 1);
            Color[] pixels = new Color[texture.Width * texture.Height];
            texture.GetData<Color>(pixels);
            DrawCenteredCircle(texture, pixels, radius, Color.White);
            texture.SetData<Color>(pixels);
            texturePosition = 
                new Vector2((int)((viewport.Width - texture.Width) / 2),
                            (int)((viewport.Height - texture.Height) / 2));
        }

        void DrawCenteredCircle(Texture2D texture, Color[] pixels, 
                                int radius, Color clr)
        {
            Point center = new Point(texture.Width / 2, texture.Height / 2);
            int halfPoint = (int)Math.Round(0.707 * radius);

            for (int y = -halfPoint; y <= halfPoint; y++)
            {
                int x1 = (int)Math.Round(Math.Sqrt(radius * radius - Math.Pow(y, 2)));
                int x2 = -x1;

                SetPixel(texture, pixels, x1 + center.X, y + center.Y, clr);
                SetPixel(texture, pixels, x2 + center.X, y + center.Y, clr);

                // Since symmetric, just swap coordinates for other piece
                SetPixel(texture, pixels, y + center.X, x1 + center.Y, clr);
                SetPixel(texture, pixels, y + center.X, x2 + center.Y, clr);
            }
        }

        void SetPixel(Texture2D texture, Color[] pixels, int x, int y, Color clr)
        {
            pixels[y * texture.Width + x] = clr;
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Navy);

            spriteBatch.Begin();
            spriteBatch.Draw(texture, texturePosition, Color.White);
            spriteBatch.DrawString(segoe14, TEXT, textPosition, Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
